; 
; Optimizing Printing by Providing a Printer Handle 
;
; One of the constructors for the Graphics class receives a device context handle and a printer handle.
; When you send Microsoft Windows GDI+ commands to certain PostScript printers, the performance will be better
; if you create your Graphics object with that particular constructor.
;
; The following console application calls GetDefaultPrinter to get the name of the default printer.
; The code passes the printer name to CreateDC to obtain a device context handle for the printer.
; The code also passes the printer name to OpenPrinter to obtain a printer handle.
; Both the device context handle and the printer handle are passed to the Graphics constructor.
; Then two figures are drawn on the printer.
;
; Note  The GetDefaultPrinter function is supported only on Windows 2000 and later.
;
; #include <windows.h>
; #include <gdiplus.h>
; #include <stdio.h>
;
; using namespace Gdiplus            <- Not sure was this does
;
Print_Test proc hWin :DWORD
    LOCAL nsize         :DWORD
    LOCAL hdcPrint      :DWORD
    LOCAL printerHandle :DWORD
    LOCAL gdiplusToken  :ULONG
    LOCAL nError        :DWORD
    LOCAL hbuffer       :DWORD
    LOCAL gdip_Token    :DWORD
    LOCAL gdip_SI       :DWORD
    LOCAL gdip_SO       :DWORD
    LOCAL rgbcolor1     :DWORD
    LOCAL hpen          :DWORD
    LOCAL hpenold       :DWORD
    LOCAL graphics      :DWORD

    invoke GdiplusStartup, ADDR gdip_Token, ADDR gdip_SI, ADDR gdip_SO
    mov dInfo.cbSize, SizeOf dInfo
    mov dInfo.lpszDocName, offset gdiplusPrintLabel
;
; GetDefaultPrinter - If NULL in 1st arg, then get the length of the printer name
;        and store it in 2nd arg.
;
    invoke GetDefaultPrinter, NULL, ADDR nsize
    .if eax == 0 && nsize == 0
        jmp Exit_Error
    .endif
    mov hbuffer, alloc(nsize + 32)                       ; allocate string space for Printer Name plus safety bytes
    invoke GetDefaultPrinter, ADDR hbuffer, ADDR nsize   ; Get the printer name.
    .if eax == 0
        jmp Exit_Error
    .else
        invoke CreateDC, NULL, ADDR hbuffer, NULL, NULL  ; Get a device context for the printer.
;
; invoke MessageBox, hWin, ADDR hbuffer, NULL, MB_OK
;  Returns SoftScanCreatePDF3 
;
        .if eax == 0
            jmp Exit_Error
        .endif
        mov hdcPrint, eax
        invoke OpenPrinter, ADDR hbuffer, ADDR printerHandle, NULL     ; Get a printer handle.
        .if eax == 0
            ;                     <------- error  here is 1801 Invalid Printer Name
            ;                              even thou messagebox shows correct name
            jmp Exit_Error
        .endif
        invoke StartDoc, hdcPrint, ADDR dInfo
        .if eax == 1
            invoke StartPage, hdcPrint
            .if eax > 0
; Graphics* graphics = new Graphics(hdcPrint, printerHandle)           < - Not sure what to do here
                invoke GdipCreateFromHDC, hdcPrint, printerHandle
                mov graphics, eax
                mov eax, 0000FF00h                                     ; Create a green pen.
                mov rgbcolor1, eax
                invoke CreatePen, PS_SOLID, 3, rgbcolor1
                mov hpen, eax
                invoke SelectObject, hDC, hpen
                mov hpenold, eax
                invoke graphics.DrawRectangle, hpen, 200, 500, 200, 150
                invoke graphics.DrawEllipse,   hpen, 200, 500, 200, 150
                invoke SelectObject, hDC, hpenold
                invoke DeleteObject, hpen
                invoke GdipDeleteGraphics, graphics
            .else
                jmp Exit_Error ; error startpage
            .endif
            invoke EndPage, hdcPrint
        .else
            jmp Exit_Error ; error startdoc
        .endif
        invoke EndDoc, hdcPrint
        invoke ClosePrinter, printerHandle
        invoke DeleteDC, hdcPrint
    .endif
    .if hbuffer > 0
        free hbuffer
    .endif
    invoke GdiplusShutdown, ADDR gdiplusToken
   return 0
Exit_Error:
   invoke GetLastError
   mov nError, eax
   .if printerHandle > 0
       invoke ClosePrinter, printerHandle
   .endif
   .if hdcPrint > 0
       invoke DeleteDC, hdcPrint
   .endif
   .if hbuffer > 0
       free hbuffer
   .endif   
;
; Get Default Printer Errors
;       if LastError == ERROR_INSUFFICIENT_BUFFER  then The pszBuffer buffer is too small. The variable pointed to by pcchBuffer contains the required buffer size, in characters.
;       if LastError == ERROR_FILE_NOT_FOUND       then There is no default printer.
;
    mov eax, nError
    ret
Print_Test endp
